#!/usr/bin/python2.7

import os
import struct
import argparse

CONFIG_CST_DIR = 'linux32'

CSF_TEMPLATE = 'tmpl.csf'
CSF_FILE     = 'csf-bin'
IVT_FILE     = 'ivt.bin'

# The name of the images that will be signed
SPL_FILE    = 'SPL'
UBOOT_FILE  = 'u-boot.img'
ZIMAGE_FILE = 'zImage'

# ivt header: tag=0xd1, len=0x0020, version=0x40
IVT_HDR = 0xd1002040

#
# Load addresses for the different images that will be signed
# The HAB ROM code will use the address when verifying an image
#
# Explanation (for imx6sx)
#                      
#   u-boot: This image is loaded by SPL and SPL will load the file 
#           at SYS_TEXT_BASE (0x87800000) minus size of image header (0x40),
#	    see common/spl.c -> spl_parse_image_header()
#
#   zImage: This image is loaded by u-boot and default load address
#           is 0x80800000 (set as u-boot environment variable)
#
loadAddr = {
	'imx6sx': {
		UBOOT_FILE : 0x877FFFC0,
		ZIMAGE_FILE : 0x80800000
	},

}

#
# append 'str' to the file name in 'file'. If the file name
# has an extension 'str' is added before the extension
#
def fappend(file, str):
	a = file.rsplit('.', 1)
	f = a[0] + str;
	if len(a) > 1:
		f = f + '.' + a[1]	
	return f

#
# Generate binary file for IVT
#
def genIvt(addr, sz):
	with open(IVT_FILE, "wb") as ivt:
		# signature
		ivt.write(struct.pack('<I', 0x402000D1))	
		# jump location (not used)
		ivt.write(struct.pack('<I', addr+0x1000))
		# reserved
		ivt.write(struct.pack('<I', 0))
		# dcd pointer
		ivt.write(struct.pack('<I', 0))
		# boot data
		ivt.write(struct.pack('<I', 0))
		# self pointer
		ivt.write(struct.pack('<I', addr+sz))
		# csf pointer (32 bytes after self/ivt)
		ivt.write(struct.pack('<I', addr+sz+0x20))
		# reserved
		ivt.write(struct.pack('<I', 0))

#
# Update CSF file with correct load address, size and name of image
#
def updateCsf(addr, sz, name):

	with open(CSF_TEMPLATE, "r") as csf:
		lines = csf.readlines()

	with open(CSF_TEMPLATE, "w") as csf:
		for line in lines:
			# the Blocks statement should contain load address
			# offset, size of image and name of image
			if line.startswith('Blocks = '):
				csf.write("Blocks = 0x%x" % addr + " 0x000 0x%x" 
					% sz + " \""+name+"\"\n")	
			else:
				csf.write(line)				

#
# Sign an image
#
def sign(file, addr, out):

	if not os.path.isfile(file):
		print "ERROR: File '{}' is missing".format(file)
		return
	
	sz = os.path.getsize(file)
	updateCsf(addr, sz, file)

	# generate CSF file
	os.system("../"+CONFIG_CST_DIR+"/cst --o "+CSF_FILE+" < " + CSF_TEMPLATE)

	# append CSF data. Now the file is 'signed'
	os.system("cat "+file+" "+CSF_FILE+" > "+out)

	# remove temporary files
	os.system("rm "+CSF_FILE)

#
# Sign an imx file, i.e., a file containing the IVT header. Typically
# an image created with mkimage (such as SPL or u-boot.imx)
#
# The header (flash_header_v2_t) is defined in tools/imximage.h
#
def signImx(file):

	if not os.path.isfile(file):
		print "ERROR: File '{}' is missing".format(file)
		return

	with open(file, "rb") as f:
		# header is 32 bytes
		hdr = f.read(32)
		# verify IVT
		ivt = struct.unpack(">I", hdr[0:4])[0]
		if ivt != IVT_HDR:
			print "{} not valid imx file {} != {}".format(file, ivt, IVT)
			return

		# the self pointer contains the address needed for signing
		selfPtr = struct.unpack("<I", hdr[20:24])[0]

	sign(file, selfPtr, fappend(file, '-signed'))


#
# PAD, add IVT, and sign an image
#
def padAndSign(file, addr):

	if not os.path.isfile(file):
		print "ERROR: File '{}' is missing".format(file)
		return
	
	sz = os.path.getsize(file)

	# the IVT must be at a 4K boundary
	szPad = (sz + 4096 - 1) & ~(4096-1)
	
	# pad to 4K boundary
	fpad = fappend(file, '-pad')
	os.system("objcopy -I binary -O binary --pad-to " + 
		hex(int(szPad)) + " --gap-fill=0x5A "+file+" "+fpad)

	# generate Initial Vector Table (IVT)
	genIvt(addr, szPad)

	# add the IVT to the end of the padded image
	fivt = fappend(file, '-ivt')
	os.system("cat "+fpad+" "+IVT_FILE+" > "+fivt)

	sign(fivt, addr, fappend(file, '-signed'))

	# remove temporary files
	os.system("rm "+fpad)
	os.system("rm "+fivt)
	os.system("rm "+IVT_FILE)

#
# main function
#
if __name__ == '__main__':

	parser = argparse.ArgumentParser()
	parser.add_argument("platform", help="Specify platform, one of: {}".format(loadAddr.keys()) )
	args = parser.parse_args()

	print "Signing {}".format(SPL_FILE)
	signImx(SPL_FILE)
	print "Signing {}".format(UBOOT_FILE)
	padAndSign(UBOOT_FILE,  loadAddr[args.platform][UBOOT_FILE])
	print "Signing {}".format(ZIMAGE_FILE)
	padAndSign(ZIMAGE_FILE, loadAddr[args.platform][ZIMAGE_FILE])





